@@ -24,11 +24,10 @@ class HistoryBookmarkModel
 
   virtual bool IsBookmarked(const GURL& url) = 0;
 
-  // Returns, by reference in |bookmarks|, the set of bookmarked urls and their
-  // titles. This returns the unique set of URLs. For example, if two bookmarks
-  // reference the same URL only one entry is added not matter the titles are
-  // same or not.
-  virtual void GetBookmarks(std::vector<UrlAndTitle>* urls) = 0;
+  // Returns the set of bookmarked urls and their titles. This returns the
+  // unique set of URLs. For example, if two bookmarks reference the same URL
+  // only one entry is added not matter the titles are same or not.
+  [[nodiscard]] virtual std::vector<UrlAndTitle> GetUniqueUrls() = 0;
 
  protected:
   friend class base::RefCountedThreadSafe<HistoryBookmarkModel>;
