@@ -10,12 +10,16 @@
 #include "base/task/sequenced_task_runner.h"
 #include "base/time/time.h"
 #include "components/cast_streaming/browser/cast_message_port_converter.h"
+#include "components/cast_streaming/browser/common/decoder_buffer_factory.h"
+#include "components/cast_streaming/browser/control/remoting/remoting_decoder_buffer_factory.h"
+#include "components/cast_streaming/browser/frame/mirroring_decoder_buffer_factory.h"
 #include "components/cast_streaming/browser/frame/stream_consumer.h"
-#include "components/cast_streaming/public/config_conversions.h"
-#include "components/cast_streaming/public/features.h"
+#include "components/cast_streaming/browser/receiver_config_conversions.h"
+#include "components/cast_streaming/common/public/features.h"
 #include "media/base/demuxer_stream.h"
 #include "media/base/media_switches.h"
 #include "media/base/timestamp_constants.h"
+#include "media/cast/openscreen/config_conversions.h"
 #include "media/mojo/common/mojo_decoder_buffer_converter.h"
 #include "mojo/public/cpp/system/data_pipe.h"
 
@@ -46,15 +50,17 @@ StreamingInitializationInfo CreateMirroringInitializationInfo(
   absl::optional<StreamingInitializationInfo::AudioStreamInfo>
       audio_stream_info;
   if (receivers.audio_receiver) {
-    audio_stream_info.emplace(ToAudioDecoderConfig(receivers.audio_config),
-                              receivers.audio_receiver);
+    audio_stream_info.emplace(
+        media::cast::ToAudioDecoderConfig(receivers.audio_config),
+        receivers.audio_receiver);
   }
 
   absl::optional<StreamingInitializationInfo::VideoStreamInfo>
       video_stream_info;
   if (receivers.video_receiver) {
-    video_stream_info.emplace(ToVideoDecoderConfig(receivers.video_config),
-                              receivers.video_receiver);
+    video_stream_info.emplace(
+        media::cast::ToVideoDecoderConfig(receivers.video_config),
+        receivers.video_receiver);
   }
 
   return {session, std::move(audio_stream_info), std::move(video_stream_info),
@@ -66,11 +72,11 @@ StreamingInitializationInfo CreateMirroringInitializationInfo(
 CastStreamingSession::ReceiverSessionClient::ReceiverSessionClient(
     CastStreamingSession::Client* client,
     absl::optional<RendererControllerConfig> renderer_controls,
-    std::unique_ptr<ReceiverSession::AVConstraints> av_constraints,
+    ReceiverConfig av_constraints,
     ReceiverSession::MessagePortProvider message_port_provider,
     scoped_refptr<base::SequencedTaskRunner> task_runner)
     : task_runner_(task_runner),
-      environment_(&openscreen::Clock::now, &task_runner_),
+      environment_(&openscreen::Clock::now, task_runner_),
       cast_message_port_converter_(CastMessagePortConverter::Create(
           std::move(message_port_provider),
           base::BindOnce(
@@ -87,14 +93,15 @@ CastStreamingSession::ReceiverSessionClient::ReceiverSessionClient(
 
   receiver_session_ = std::make_unique<openscreen::cast::ReceiverSession>(
       this, &environment_, &cast_message_port_converter_->GetMessagePort(),
-      std::move(*av_constraints));
+      ToOpenscreenConstraints(av_constraints));
 
   if (renderer_controls) {
     playback_command_dispatcher_ = std::make_unique<PlaybackCommandDispatcher>(
         task_runner, std::move(renderer_controls.value().control_configuration),
         base::BindRepeating(
             &CastStreamingSession::ReceiverSessionClient::OnFlushUntil,
-            weak_factory_.GetWeakPtr()));
+            weak_factory_.GetWeakPtr()),
+        std::move(av_constraints.remoting));
     playback_command_dispatcher_->RegisterCommandSource(
         std::move(renderer_controls.value().external_renderer_controls));
   }
@@ -182,18 +189,27 @@ CastStreamingSession::ReceiverSessionClient::InitializeAudioConsumer(
     return absl::nullopt;
   }
 
+  std::unique_ptr<DecoderBufferFactory> decoder_buffer_factory;
+  if (initialization_info.is_remoting) {
+    decoder_buffer_factory = std::make_unique<RemotingDecoderBufferFactory>();
+  } else {
+    // The duration is set to kNoTimestamp so the audio renderer does not block.
+    // Audio frames duration is not known ahead of time in mirroring.
+    decoder_buffer_factory = std::make_unique<MirroringDecoderBufferFactory>(
+        initialization_info.audio_stream_info->receiver->rtp_timebase(),
+        media::kNoTimestamp);
+  }
+
   // We can use unretained pointers here because StreamConsumer is owned by
-  // this object and |client_| is guaranteed to outlive this object. Here,
-  // the duration is set to kNoTimestamp so the audio renderer does not block.
-  // Audio frames duration is not known ahead of time in mirroring.
+  // this object and |client_| is guaranteed to outlive this object.
   audio_consumer_ = std::make_unique<StreamConsumer>(
-      initialization_info.audio_stream_info->receiver, media::kNoTimestamp,
+      initialization_info.audio_stream_info->receiver,
       std::move(data_pipe_producer),
       base::BindRepeating(&CastStreamingSession::Client::OnAudioBufferReceived,
                           base::Unretained(client_)),
       base::BindRepeating(&base::OneShotTimer::Reset,
                           base::Unretained(&data_timeout_timer_)),
-      initialization_info.is_remoting);
+      std::move(decoder_buffer_factory));
 
   return data_pipe_consumer;
 }
@@ -211,22 +227,31 @@ CastStreamingSession::ReceiverSessionClient::InitializeVideoConsumer(
     return absl::nullopt;
   }
 
+  std::unique_ptr<DecoderBufferFactory> decoder_buffer_factory;
+  if (initialization_info.is_remoting) {
+    decoder_buffer_factory = std::make_unique<RemotingDecoderBufferFactory>();
+  } else {
+    // The frame duration is set to 10 minutes to work around cases where
+    // senders do not send data for a long period of time. We end up with
+    // overlapping video frames but this is fine since the media pipeline mostly
+    // considers the playout time when deciding which frame to present or play
+    decoder_buffer_factory = std::make_unique<MirroringDecoderBufferFactory>(
+        initialization_info.video_stream_info->receiver->rtp_timebase(),
+        base::Minutes(10));
+  }
+
   // We can use unretained pointers here because StreamConsumer is owned by
   // this object and |client_| is guaranteed to outlive this object.
   // |data_timeout_timer_| is also owned by this object and will outlive both
   // StreamConsumers.
-  // The frame duration is set to 10 minutes to work around cases where
-  // senders do not send data for a long period of time. We end up with
-  // overlapping video frames but this is fine since the media pipeline mostly
-  // considers the playout time when deciding which frame to present or play
   video_consumer_ = std::make_unique<StreamConsumer>(
-      initialization_info.video_stream_info->receiver, base::Minutes(10),
+      initialization_info.video_stream_info->receiver,
       std::move(data_pipe_producer),
       base::BindRepeating(&CastStreamingSession::Client::OnVideoBufferReceived,
                           base::Unretained(client_)),
       base::BindRepeating(&base::OneShotTimer::Reset,
                           base::Unretained(&data_timeout_timer_)),
-      initialization_info.is_remoting);
+      std::move(decoder_buffer_factory));
 
   return data_pipe_consumer;
 }
@@ -235,6 +260,7 @@ void CastStreamingSession::ReceiverSessionClient::StartStreamingSession(
     StreamingInitializationInfo initialization_info) {
   DVLOG(1) << __func__;
   DCHECK_EQ(initialization_info.session, receiver_session_.get());
+  DCHECK(!initialization_info.is_remoting || IsCastRemotingEnabled());
 
   // If a Flush() call is ongoing, its unsafe to begin streaming data, so
   // instead stall this call until the Flush() call has completed.
@@ -440,7 +466,7 @@ CastStreamingSession::~CastStreamingSession() = default;
 void CastStreamingSession::Start(
     Client* client,
     absl::optional<RendererControllerConfig> renderer_controls,
-    std::unique_ptr<ReceiverSession::AVConstraints> av_constraints,
+    ReceiverConfig av_constraints,
     ReceiverSession::MessagePortProvider message_port_provider,
     scoped_refptr<base::SequencedTaskRunner> task_runner) {
   DVLOG(1) << __func__;
