@@ -86,6 +86,11 @@ SupportedCodecs GetHevcCodecs(
     return media::EME_CODEC_NONE;
   }
 #endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  if (!base::FeatureList::IsEnabled(media::kPlatformHEVCDecoderSupport)) {
+    return media::EME_CODEC_NONE;
+  }
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
   // If no profiles are specified, then all are supported.
   if (profiles.empty()) {
@@ -131,9 +136,6 @@ SupportedCodecs GetDolbyVisionCodecs(
       case media::DOLBYVISION_PROFILE0:
         supported_dv_codecs |= media::EME_CODEC_DOLBY_VISION_PROFILE0;
         break;
-      case media::DOLBYVISION_PROFILE4:
-        supported_dv_codecs |= media::EME_CODEC_DOLBY_VISION_PROFILE4;
-        break;
       case media::DOLBYVISION_PROFILE5:
         supported_dv_codecs |= media::EME_CODEC_DOLBY_VISION_PROFILE5;
         break;
@@ -184,6 +186,19 @@ SupportedCodecs GetSupportedCodecs(const media::CdmCapability& capability,
         supported_codecs |= media::EME_CODEC_DTSXP2;
         break;
 #endif  // BUILDFLAG(ENABLE_PLATFORM_DTS_AUDIO)
+#if BUILDFLAG(ENABLE_PLATFORM_AC3_EAC3_AUDIO)
+      case media::AudioCodec::kAC3:
+        supported_codecs |= media::EME_CODEC_AC3;
+        break;
+      case media::AudioCodec::kEAC3:
+        supported_codecs |= media::EME_CODEC_EAC3;
+        break;
+#endif  // BUILDFLAG(ENABLE_PLATFORM_AC3_EAC3_AUDIO)
+#if BUILDFLAG(ENABLE_PLATFORM_AC4_AUDIO)
+      case media::AudioCodec::kAC4:
+        supported_codecs |= media::EME_CODEC_AC4;
+        break;
+#endif  // BUILDFLAG(ENABLE_PLATFORM_AC4_AUDIO)
 #endif  // BUILDFLAG(USE_PROPRIETARY_CODECS)
       default:
         DVLOG(1) << "Unexpected supported codec: " << GetCodecName(codec);
@@ -472,12 +487,15 @@ void AddMediaFoundationClearKey(
   }
 
 #if BUILDFLAG(USE_PROPRIETARY_CODECS)
+  // TODO(crbug.com/1412485): Remove this hard-codeded supported codecs so that
+  // real hardware capabilities can be checked.
   key_systems->push_back(std::make_unique<ExternalClearKeyKeySystemInfo>(
       media::kMediaFoundationClearKeyKeySystem, std::vector<std::string>(),
       // MediaFoundation Clear Key Key System uses Windows Media Foundation's
-      // decoders and H264 is always supported. VideoCodec::kH264 is an
-      // EME_CODEC_AVC1.
-      media::EME_CODEC_AVC1,
+      // decoders. H264 ("avc1.64001E") for video and MP4 AAC ("mp4a.40.2") for
+      // audio are always supported. VideoCodec::kH264 is an EME_CODEC_AVC1.
+      // AudioCodec::kAAC is an EME_CODEC_AAC.
+      media::EME_CODEC_AVC1 | media::EME_CODEC_AAC,
       // On Windows, MediaFoundation Clear Key CDM requires identifier,
       // persistent state and HW secure codecs. We pretent to require these for
       // testing purposes.
