@@ -626,20 +626,20 @@ void JpegClient::SaveToFile(TestImage* test_image,
   LOG(INFO) << "Writing HW encode results to "
             << out_filename_hw.MaybeAsASCII();
 
-  ASSERT_EQ(static_cast<int>(hw_size),
-            base::WriteFile(out_filename_hw,
-                            static_cast<const char*>(
-                                hw_out_frame_ ? hw_out_frame_->data(0)
-                                              : hw_out_mapping_.memory()),
-                            hw_size));
+  ASSERT_TRUE(base::WriteFile(
+      out_filename_hw,
+      base::make_span(hw_out_frame_
+                          ? hw_out_frame_->data(0)
+                          : static_cast<uint8_t*>(hw_out_mapping_.memory()),
+                      hw_size)));
 
   base::FilePath out_filename_sw = out_filename_hw.InsertBeforeExtension("_sw");
   LOG(INFO) << "Writing SW encode results to "
             << out_filename_sw.MaybeAsASCII();
-  ASSERT_EQ(
-      static_cast<int>(sw_size),
-      base::WriteFile(out_filename_sw,
-                      static_cast<char*>(sw_out_mapping_.memory()), sw_size));
+  ASSERT_TRUE(base::WriteFile(
+      out_filename_sw,
+      base::make_span(static_cast<uint8_t*>(sw_out_mapping_.memory()),
+                      sw_size)));
 }
 
 void JpegClient::StartEncode(int32_t bitstream_buffer_id) {
