@@ -18,6 +18,10 @@ namespace base {
 class Value;
 }  // namespace base
 
+namespace content {
+class RenderFrameHost;
+}  // namespace content
+
 namespace commerce {
 
 // A WebWrapper backed by content::WebContents.
@@ -27,23 +31,36 @@ class WebContentsWrapper : public WebWrapper {
                               int32_t js_world_id);
   WebContentsWrapper(const WebContentsWrapper&) = delete;
   WebContentsWrapper operator=(const WebContentsWrapper&) = delete;
-  ~WebContentsWrapper() override = default;
+  ~WebContentsWrapper() override;
 
   const GURL& GetLastCommittedURL() override;
 
+  bool IsFirstLoadForNavigationFinished() override;
+
+  void SetIsFirstLoadForNavigationFinished(bool finished);
+
   bool IsOffTheRecord() override;
 
   void RunJavascript(
       const std::u16string& script,
       base::OnceCallback<void(const base::Value)> callback) override;
 
+  ukm::SourceId GetPageUkmSourceId() override;
+
   void ClearWebContentsPointer();
 
+  content::RenderFrameHost* GetPrimaryMainFrame();
+
  private:
-  base::raw_ptr<content::WebContents> web_contents_;
+  raw_ptr<content::WebContents> web_contents_;
 
   // The ID of the isolated world to run javascript in.
   int32_t js_world_id_;
+
+  // Whether the first load after a navigation has completed. This is useful
+  // when dealing with single-page apps that may not fire subsequent load
+  // events.
+  bool is_first_load_for_nav_finished_{false};
 };
 
 }  // namespace commerce
