@@ -7,11 +7,11 @@
 namespace commerce {
 
 MockAccountChecker::MockAccountChecker()
-    : AccountChecker(nullptr, nullptr, nullptr) {
+    : AccountChecker(nullptr, nullptr, nullptr, nullptr) {
   // Default to an account checker with the fewest restrictions.
   SetSignedIn(true);
+  SetSyncingBookmarks(true);
   SetAnonymizedUrlDataCollectionEnabled(true);
-  SetWebAndAppActivityEnabled(true);
   SetIsSubjectToParentalControls(false);
 }
 
@@ -21,13 +21,12 @@ void MockAccountChecker::SetSignedIn(bool signed_in) {
   ON_CALL(*this, IsSignedIn).WillByDefault(testing::Return(signed_in));
 }
 
-void MockAccountChecker::SetAnonymizedUrlDataCollectionEnabled(bool enabled) {
-  ON_CALL(*this, IsAnonymizedUrlDataCollectionEnabled)
-      .WillByDefault(testing::Return(enabled));
+void MockAccountChecker::SetSyncingBookmarks(bool syncing) {
+  ON_CALL(*this, IsSyncingBookmarks).WillByDefault(testing::Return(syncing));
 }
 
-void MockAccountChecker::SetWebAndAppActivityEnabled(bool enabled) {
-  ON_CALL(*this, IsWebAndAppActivityEnabled)
+void MockAccountChecker::SetAnonymizedUrlDataCollectionEnabled(bool enabled) {
+  ON_CALL(*this, IsAnonymizedUrlDataCollectionEnabled)
       .WillByDefault(testing::Return(enabled));
 }
 
