@@ -34,9 +34,8 @@ class ContentCaptureReceiverTest : public content::RenderViewHostTestHarness,
   void SetUp() override {
     if (GetParam()) {
       scoped_feature_list_.InitWithFeaturesAndParameters(
-          {{{features::kBackForwardCache}, {}}},
-          // Allow BackForwardCache for all devices regardless of their memory.
-          {features::kBackForwardCacheMemoryControls});
+          content::GetBasicBackForwardCacheFeatureForTesting(),
+          content::GetDefaultDisabledBackForwardCacheFeaturesForTesting());
     }
     content::RenderViewHostTestHarness::SetUp();
     helper_.CreateProviderAndConsumer(web_contents(),
@@ -123,8 +122,8 @@ class ContentCaptureReceiverTest : public content::RenderViewHostTestHarness,
   // The sender for child frame.
   std::unique_ptr<FakeContentCaptureSender> child_frame_sender_;
 
-  raw_ptr<content::RenderFrameHost> main_frame_ = nullptr;
-  raw_ptr<content::RenderFrameHost> child_frame_ = nullptr;
+  raw_ptr<content::RenderFrameHost, DanglingUntriaged> main_frame_ = nullptr;
+  raw_ptr<content::RenderFrameHost, DanglingUntriaged> child_frame_ = nullptr;
 
   // Expected removed Ids.
   std::vector<int64_t> expected_removed_ids_{2};
@@ -407,10 +406,29 @@ TEST_P(ContentCaptureReceiverTest, MAYBE_ChildFrameCaptureContentFirst) {
                     &removed_child_session);
   ContentCaptureSession removed_main_session = expected;
   // When main frame navigates to same url, the parent session will not change.
+  bool rfh_should_change =
+      web_contents()
+          ->GetPrimaryMainFrame()
+          ->ShouldChangeRenderFrameHostOnSameSiteNavigation();
   NavigateMainFrame(GURL(kMainFrameUrl));
   SetupChildFrame();
   child_frame_sender()->DidCaptureContent(helper()->test_data2(),
                                           true /* first_data */);
+
+  // Intentionally reuse the data.id from previous result, so we know navigating
+  // to same domain didn't create new ContentCaptureReceiver when call
+  // VerifySession(), otherwise, we can't test the code to handle the navigation
+  // in ContentCaptureReceiver - except when RenderDocument is enabled, where we
+  // will get new RenderFrameHosts after the navigation to |kMainFrameUrl|.
+  if (rfh_should_change) {
+    data = GetExpectedTestData(helper()->test_data(),
+                               GetFrameId(true /* main_frame */));
+  }
+  data.url = kMainFrameUrl;
+  // Currently, there is no way to fake frame size, set it to 0.
+  data.bounds = gfx::Rect();
+  expected.clear();
+  expected.push_back(data);
   VerifySession(expected, consumer()->parent_session());
 
   EXPECT_EQ(2u, consumer()->removed_sessions().size());
@@ -509,14 +527,15 @@ TEST_P(ContentCaptureReceiverTest, ConvertFaviconURLToJSON) {
   EXPECT_TRUE(ContentCaptureReceiver::ToJSON(favicon_urls).empty());
   favicon_urls.push_back(blink::mojom::FaviconURL::New(
       GURL{"https://a.com"}, blink::mojom::FaviconIconType::kFavicon,
-      std::vector<gfx::Size>{gfx::Size(10, 10)}));
+      std::vector<gfx::Size>{gfx::Size(10, 10)}, /*is_default_icon=*/false));
   favicon_urls.push_back(blink::mojom::FaviconURL::New(
       GURL{"https://b.com"}, blink::mojom::FaviconIconType::kTouchIcon,
-      std::vector<gfx::Size>{gfx::Size(100, 100), gfx::Size(20, 20)}));
+      std::vector<gfx::Size>{gfx::Size(100, 100), gfx::Size(20, 20)},
+      /*is_default_icon=*/false));
   favicon_urls.push_back(blink::mojom::FaviconURL::New(
       GURL{"https://c.com"},
       blink::mojom::FaviconIconType::kTouchPrecomposedIcon,
-      std::vector<gfx::Size>{}));
+      std::vector<gfx::Size>{}, /*is_default_icon=*/false));
   std::string actual_json = ContentCaptureReceiver::ToJSON(favicon_urls);
   absl::optional<base::Value> actual = base::JSONReader::Read(actual_json);
   std::string expected_json =
