@@ -444,7 +444,8 @@ void CrashHandlerHostLinux::WriteDumpFile(BreakpadInfo* info,
   // Create a temporary file holding the AddressSanitizer report.
   const base::FilePath log_path =
       base::FilePath(minidump_filename).ReplaceExtension("log");
-  base::WriteFile(log_path, info->asan_report_str, info->asan_report_length);
+  base::WriteFile(log_path, base::StringPiece(info->asan_report_str,
+                                              info->asan_report_length));
 #endif
 
   // Freed in CrashDumpTask().
